@@ -1,4 +1,3 @@
-/* $Id: config.h 5791 2018-05-14 01:24:43Z ming $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -54,7 +53,7 @@
  * Settings to enable L16 codec 8KHz, mono. By default it is disabled.
  */
 #ifndef PJMEDIA_CODEC_L16_HAS_8KHZ_MONO
-#   define PJMEDIA_CODEC_L16_HAS_8KHZ_MONO	0
+#   define PJMEDIA_CODEC_L16_HAS_8KHZ_MONO      0
 #endif
 
 
@@ -62,7 +61,7 @@
  * Settings to enable L16 codec 8KHz, stereo. By default it is disabled.
  */
 #ifndef PJMEDIA_CODEC_L16_HAS_8KHZ_STEREO
-#   define PJMEDIA_CODEC_L16_HAS_8KHZ_STEREO	0
+#   define PJMEDIA_CODEC_L16_HAS_8KHZ_STEREO    0
 #endif
 
 
@@ -70,7 +69,7 @@
  * Settings to enable L16 codec 16KHz, mono. By default it is disabled.
  */
 #ifndef PJMEDIA_CODEC_L16_HAS_16KHZ_MONO
-#   define PJMEDIA_CODEC_L16_HAS_16KHZ_MONO	0
+#   define PJMEDIA_CODEC_L16_HAS_16KHZ_MONO     0
 #endif
 
 
@@ -78,7 +77,7 @@
  * Settings to enable L16 codec 16KHz, stereo. By default it is disabled.
  */
 #ifndef PJMEDIA_CODEC_L16_HAS_16KHZ_STEREO
-#   define PJMEDIA_CODEC_L16_HAS_16KHZ_STEREO	0
+#   define PJMEDIA_CODEC_L16_HAS_16KHZ_STEREO   0
 #endif
 
 
@@ -86,7 +85,7 @@
  * Settings to enable L16 codec 48KHz, mono. By default it is disabled.
  */
 #ifndef PJMEDIA_CODEC_L16_HAS_48KHZ_MONO
-#   define PJMEDIA_CODEC_L16_HAS_48KHZ_MONO	0
+#   define PJMEDIA_CODEC_L16_HAS_48KHZ_MONO     0
 #endif
 
 
@@ -94,7 +93,7 @@
  * Settings to enable L16 codec 48KHz, stereo. By default it is disabled.
  */
 #ifndef PJMEDIA_CODEC_L16_HAS_48KHZ_STEREO
-#   define PJMEDIA_CODEC_L16_HAS_48KHZ_STEREO	0
+#   define PJMEDIA_CODEC_L16_HAS_48KHZ_STEREO   0
 #endif
 
 
@@ -125,7 +124,7 @@
  * this setting via its codec quality setting (i.e PJSUA_DEFAULT_CODEC_QUALITY).
  */
 #ifndef PJMEDIA_CODEC_SPEEX_DEFAULT_QUALITY
-#   define PJMEDIA_CODEC_SPEEX_DEFAULT_QUALITY	    8
+#   define PJMEDIA_CODEC_SPEEX_DEFAULT_QUALITY      8
 #endif
 
 
@@ -144,6 +143,19 @@
 #   define PJMEDIA_HAS_G722_CODEC    1
 #endif
 
+/**
+ * Initial memory block for G.722 codec implementation.
+ */
+#ifndef PJMEDIA_POOL_LEN_G722_CODEC
+#   define PJMEDIA_POOL_LEN_G722_CODEC  1000
+#endif
+
+/**
+ * Memory increment for G.722 codec implementation.
+ */
+#ifndef PJMEDIA_POOL_INC_G722_CODEC
+#   define PJMEDIA_POOL_INC_G722_CODEC  1000
+#endif
 
 /**
  * Default G.722 codec encoder and decoder level adjustment. The G.722
@@ -159,7 +171,7 @@
  * Default: 2.
  */
 #ifndef PJMEDIA_G722_DEFAULT_PCM_SHIFT
-#   define PJMEDIA_G722_DEFAULT_PCM_SHIFT	    2
+#   define PJMEDIA_G722_DEFAULT_PCM_SHIFT           2
 #endif
 
 
@@ -188,7 +200,7 @@
  * Intel IPP with PJMEDIA.
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP
-#   define PJMEDIA_HAS_INTEL_IPP		0
+#   define PJMEDIA_HAS_INTEL_IPP                0
 #endif
 
 
@@ -207,7 +219,7 @@
  * Default 1.
  */
 #ifndef PJMEDIA_AUTO_LINK_IPP_LIBS
-#   define PJMEDIA_AUTO_LINK_IPP_LIBS		1
+#   define PJMEDIA_AUTO_LINK_IPP_LIBS           1
 #endif
 
 
@@ -219,7 +231,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP_CODEC_AMR
-#   define PJMEDIA_HAS_INTEL_IPP_CODEC_AMR	1
+#   define PJMEDIA_HAS_INTEL_IPP_CODEC_AMR      1
 #endif
 
 
@@ -231,7 +243,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP_CODEC_AMRWB
-#   define PJMEDIA_HAS_INTEL_IPP_CODEC_AMRWB	1
+#   define PJMEDIA_HAS_INTEL_IPP_CODEC_AMRWB    1
 #endif
 
 
@@ -242,7 +254,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP_CODEC_G729
-#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G729	1
+#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G729     1
 #endif
 
 
@@ -253,7 +265,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP_CODEC_G723_1
-#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G723_1	1
+#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G723_1   1
 #endif
 
 
@@ -264,7 +276,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP_CODEC_G726
-#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G726	1
+#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G726     1
 #endif
 
 
@@ -275,7 +287,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP_CODEC_G728
-#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G728	1
+#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G728     1
 #endif
 
 
@@ -286,7 +298,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_INTEL_IPP_CODEC_G722_1
-#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G722_1	1
+#   define PJMEDIA_HAS_INTEL_IPP_CODEC_G722_1   1
 #endif
 
 /**
@@ -295,7 +307,7 @@
  * Default: 0
  */
 #ifndef PJMEDIA_HAS_PASSTHROUGH_CODECS
-#   define PJMEDIA_HAS_PASSTHROUGH_CODECS	0
+#   define PJMEDIA_HAS_PASSTHROUGH_CODECS       0
 #endif
 
 /**
@@ -304,7 +316,16 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_PASSTHROUGH_CODEC_AMR
-#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_AMR	1
+#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_AMR    1
+#endif
+
+/**
+ * Enable G.722 passthrough codec.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_HAS_PASSTHROUGH_CODEC_G722
+#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_G722   1
 #endif
 
 /**
@@ -313,7 +334,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_PASSTHROUGH_CODEC_G729
-#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_G729	1
+#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_G729   1
 #endif
 
 /**
@@ -322,7 +343,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_PASSTHROUGH_CODEC_ILBC
-#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_ILBC	1
+#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_ILBC   1
 #endif
 
 /**
@@ -331,7 +352,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMU
-#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMU	1
+#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMU   1
 #endif
 
 /**
@@ -340,7 +361,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMA
-#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMA	1
+#   define PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMA   1
 #endif
 
 /* If passthrough and PCMU/PCMA are enabled, disable the software
@@ -349,7 +370,7 @@
 #if PJMEDIA_HAS_PASSTHROUGH_CODECS && \
     (PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMU || PJMEDIA_HAS_PASSTHROUGH_CODEC_PCMA)
 #   undef PJMEDIA_HAS_G711_CODEC
-#   define PJMEDIA_HAS_G711_CODEC		0
+#   define PJMEDIA_HAS_G711_CODEC               0
 #endif
 
 
@@ -357,27 +378,35 @@
  * G.722.1 codec is disabled by default.
  */
 #ifndef PJMEDIA_HAS_G7221_CODEC
-#   define PJMEDIA_HAS_G7221_CODEC		0
+#   define PJMEDIA_HAS_G7221_CODEC              0
+#endif
+
+/* If passthrough and G.722 is enabled, disable the software
+ * G.722 codec
+ */
+#if PJMEDIA_HAS_PASSTHROUGH_CODECS && PJMEDIA_HAS_PASSTHROUGH_CODEC_G722
+#   undef PJMEDIA_HAS_G722_CODEC
+#   define PJMEDIA_HAS_G722_CODEC               0
 #endif
 
 /**
  * Enable OpenCORE AMR-NB codec.
- * See https://trac.pjsip.org/repos/ticket/1388 for some info.
+ * See https://github.com/pjsip/pjproject/issues/1388 for some info.
  *
  * Default: 0
  */
 #ifndef PJMEDIA_HAS_OPENCORE_AMRNB_CODEC
-#   define PJMEDIA_HAS_OPENCORE_AMRNB_CODEC	0
+#   define PJMEDIA_HAS_OPENCORE_AMRNB_CODEC     0
 #endif
 
 /**
  * Enable OpenCORE AMR-WB codec.
- * See https://trac.pjsip.org/repos/ticket/1608 for some info.
+ * See https://github.com/pjsip/pjproject/issues/1608 for some info.
  *
  * Default: 0
  */
 #ifndef PJMEDIA_HAS_OPENCORE_AMRWB_CODEC
-#   define PJMEDIA_HAS_OPENCORE_AMRWB_CODEC	0
+#   define PJMEDIA_HAS_OPENCORE_AMRWB_CODEC     0
 #endif
 
 /**
@@ -388,7 +417,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_AUTO_LINK_OPENCORE_AMR_LIBS
-#  define PJMEDIA_AUTO_LINK_OPENCORE_AMR_LIBS	1
+#  define PJMEDIA_AUTO_LINK_OPENCORE_AMR_LIBS   1
 #endif
 
 /**
@@ -399,7 +428,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_OPENCORE_AMR_BUILT_WITH_GCC
-#   define PJMEDIA_OPENCORE_AMR_BUILT_WITH_GCC	1
+#   define PJMEDIA_OPENCORE_AMR_BUILT_WITH_GCC  1
 #endif
 
 
@@ -413,7 +442,7 @@
  * #pjmedia_codec_g7221_set_pcm_shift().
  */
 #ifndef PJMEDIA_G7221_DEFAULT_PCM_SHIFT
-#   define PJMEDIA_G7221_DEFAULT_PCM_SHIFT	1
+#   define PJMEDIA_G7221_DEFAULT_PCM_SHIFT      1
 #endif
 
 
@@ -429,9 +458,9 @@
 #if (PJMEDIA_HAS_G7221_CODEC != 0) && (PJMEDIA_HAS_INTEL_IPP != 0) && \
     (PJMEDIA_HAS_INTEL_IPP_CODEC_G722_1 != 0)
 #   error Only one G.722.1 implementation can be enabled at the same time. \
-	  Please use PJMEDIA_HAS_G7221_CODEC and \
-	  PJMEDIA_HAS_INTEL_IPP_CODEC_G722_1 in your config_site.h \
-	  to control which implementation to be used.
+          Please use PJMEDIA_HAS_G7221_CODEC and \
+          PJMEDIA_HAS_INTEL_IPP_CODEC_G722_1 in your config_site.h \
+          to control which implementation to be used.
 #endif
 
 
@@ -441,7 +470,7 @@
  * Default: 0
  */
 #ifndef PJMEDIA_HAS_SILK_CODEC
-#   define PJMEDIA_HAS_SILK_CODEC		0
+#   define PJMEDIA_HAS_SILK_CODEC               0
 #endif
 
 
@@ -463,7 +492,7 @@
  * Default: 10
  */
 #ifndef PJMEDIA_CODEC_SILK_DEFAULT_QUALITY
-#   define PJMEDIA_CODEC_SILK_DEFAULT_QUALITY	    10
+#   define PJMEDIA_CODEC_SILK_DEFAULT_QUALITY       10
 #endif
 
 
@@ -473,7 +502,7 @@
  * Default: 0
  */
 #ifndef PJMEDIA_HAS_OPUS_CODEC
-#   define PJMEDIA_HAS_OPUS_CODEC			0
+#   define PJMEDIA_HAS_OPUS_CODEC                       0
 #endif
 
 /**
@@ -482,7 +511,7 @@
  * Default: 48000
  */
 #ifndef PJMEDIA_CODEC_OPUS_DEFAULT_SAMPLE_RATE
-#   define PJMEDIA_CODEC_OPUS_DEFAULT_SAMPLE_RATE  	48000
+#   define PJMEDIA_CODEC_OPUS_DEFAULT_SAMPLE_RATE       48000
 #endif
 
 /**
@@ -493,7 +522,7 @@
  * rate, channel count, etc).
  */
 #ifndef PJMEDIA_CODEC_OPUS_DEFAULT_BIT_RATE
-#   define PJMEDIA_CODEC_OPUS_DEFAULT_BIT_RATE  	0
+#   define PJMEDIA_CODEC_OPUS_DEFAULT_BIT_RATE          0
 #endif
 
 
@@ -504,7 +533,7 @@
  * Default: 5
  */
 #ifndef PJMEDIA_CODEC_OPUS_DEFAULT_COMPLEXITY
-#   define PJMEDIA_CODEC_OPUS_DEFAULT_COMPLEXITY 	5
+#   define PJMEDIA_CODEC_OPUS_DEFAULT_COMPLEXITY        5
 #endif
 
 
@@ -514,7 +543,7 @@
  * Default: PJ_FALSE (which means Opus will use VBR (variable bit rate))
  */
 #ifndef PJMEDIA_CODEC_OPUS_DEFAULT_CBR
-#   define PJMEDIA_CODEC_OPUS_DEFAULT_CBR 		PJ_FALSE
+#   define PJMEDIA_CODEC_OPUS_DEFAULT_CBR               PJ_FALSE
 #endif
 
 
@@ -524,17 +553,86 @@
  * Default: 0 
  */
 #ifndef PJMEDIA_HAS_BCG729
-#   define PJMEDIA_HAS_BCG729				0
+#   define PJMEDIA_HAS_BCG729                           0
 #endif
 
 
+ /**
+  * Enable Lyra codec.
+  *
+  * Default: 0
+  */
+#ifndef PJMEDIA_HAS_LYRA_CODEC
+#   define PJMEDIA_HAS_LYRA_CODEC                       0
+#endif
+
+/**
+ * Lyra default bitrate setting
+ *
+ * Default: 3200 (available bitrate:3200, 6000, 9200)
+ */
+#ifndef PJMEDIA_CODEC_LYRA_DEFAULT_BIT_RATE
+#   define PJMEDIA_CODEC_LYRA_DEFAULT_BIT_RATE           3200
+#endif
+
+/**
+ * Lyra default model path containing lyra_config.binarypb, lyragan.tflite,
+ * quantizer.tflite and soundstream_encoder.tflite file. If autoconf is used,
+ * it will be set to "[lyra src folder]/model_coeffs".
+ *
+ * Default: "model_coeffs"
+ */
+#ifndef PJMEDIA_CODEC_LYRA_DEFAULT_MODEL_PATH
+#   define PJMEDIA_CODEC_LYRA_DEFAULT_MODEL_PATH           "model_coeffs"
+#endif
+
+/**
+ * Settings to enable Lyra codec 8KHz. This option is only used
+ * when PJMEDIA_HAS_LYRA_CODEC is enabled.
+ *
+ * Default: 0
+ */
+#ifndef PJMEDIA_CODEC_LYRA_HAS_8KHZ
+#   define PJMEDIA_CODEC_LYRA_HAS_8KHZ   0
+#endif
+
+/**
+ * Settings to enable Lyra codec 16KHz. This option is only used
+ * when PJMEDIA_HAS_LYRA_CODEC is enabled.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_CODEC_LYRA_HAS_16KHZ
+#   define PJMEDIA_CODEC_LYRA_HAS_16KHZ   1
+#endif
+
+/**
+ * Settings to enable Lyra codec 32KHz. This option is only used
+ * when PJMEDIA_HAS_LYRA_CODEC is enabled.
+ *
+ * Default: 0
+ */
+#ifndef PJMEDIA_CODEC_LYRA_HAS_32KHZ
+#   define PJMEDIA_CODEC_LYRA_HAS_32KHZ   0
+#endif
+
+/**
+ * Settings to enable Lyra codec 48KHz. This option is only used
+ * when PJMEDIA_HAS_LYRA_CODEC is enabled.
+ *
+ * Default: 0
+ */
+#ifndef PJMEDIA_CODEC_LYRA_HAS_48KHZ
+#   define PJMEDIA_CODEC_LYRA_HAS_48KHZ   0
+#endif
+
 /**
  * Specify if FFMPEG codecs are available.
  *
  * Default: PJMEDIA_HAS_LIBAVFORMAT
  */
 #ifndef PJMEDIA_HAS_FFMPEG_CODEC
-#   define PJMEDIA_HAS_FFMPEG_CODEC		PJMEDIA_HAS_LIBAVFORMAT
+#   define PJMEDIA_HAS_FFMPEG_CODEC             PJMEDIA_HAS_LIBAVFORMAT
 #endif
 
 
@@ -544,7 +642,7 @@
  * Default: PJMEDIA_HAS_FFMPEG_CODEC
  */
 #ifndef PJMEDIA_HAS_FFMPEG_VID_CODEC
-#   define PJMEDIA_HAS_FFMPEG_VID_CODEC		PJMEDIA_HAS_FFMPEG_CODEC
+#   define PJMEDIA_HAS_FFMPEG_VID_CODEC         PJMEDIA_HAS_FFMPEG_CODEC
 #endif
 
 /**
@@ -553,7 +651,7 @@
  * Default: 1
  */
 #ifndef PJMEDIA_HAS_FFMPEG_CODEC_H263P
-#   define PJMEDIA_HAS_FFMPEG_CODEC_H263P	PJMEDIA_HAS_FFMPEG_VID_CODEC
+#   define PJMEDIA_HAS_FFMPEG_CODEC_H263P       PJMEDIA_HAS_FFMPEG_VID_CODEC
 #endif
 
 /**
@@ -564,16 +662,145 @@
  */
 #ifndef PJMEDIA_HAS_FFMPEG_CODEC_H264
 #   if defined(PJMEDIA_HAS_OPENH264_CODEC) && PJMEDIA_HAS_OPENH264_CODEC != 0
-#	define PJMEDIA_HAS_FFMPEG_CODEC_H264	0
+#       define PJMEDIA_HAS_FFMPEG_CODEC_H264    0
 #   else
-#	define PJMEDIA_HAS_FFMPEG_CODEC_H264	PJMEDIA_HAS_FFMPEG_VID_CODEC
+#       define PJMEDIA_HAS_FFMPEG_CODEC_H264    PJMEDIA_HAS_FFMPEG_VID_CODEC
 #   endif
 #endif
 
+/**
+ * Enable FFMPEG VPX codec (requires libvpx)
+ */
+#ifndef PJMEDIA_HAS_FFMPEG_CODEC_VP8
+#   if defined(PJMEDIA_HAS_VPX_CODEC) && PJMEDIA_HAS_VPX_CODEC != 0
+#       define PJMEDIA_HAS_FFMPEG_CODEC_VP8             0
+#   else
+#       define PJMEDIA_HAS_FFMPEG_CODEC_VP8             1
+#   endif
+#endif
+
+#ifndef PJMEDIA_HAS_FFMPEG_CODEC_VP9
+#   if defined(PJMEDIA_HAS_VPX_CODEC) && PJMEDIA_HAS_VPX_CODEC != 0
+#       define PJMEDIA_HAS_FFMPEG_CODEC_VP9             0
+#   else
+#       define PJMEDIA_HAS_FFMPEG_CODEC_VP9             1
+#   endif
+#endif
+
+
+/**
+ * Determine the log level of the native openH264 log which will be forwarded
+ * to the library's log.
+ * Set to WELS_LOG_QUIET to disable logging, or WELS_LOG_DETAIL for debugging.
+ *
+ * Default: WELS_LOG_ERROR.
+ */
+#ifndef PJMEDIA_CODEC_OPENH264_LOG_LEVEL
+#   define PJMEDIA_CODEC_OPENH264_LOG_LEVEL  WELS_LOG_ERROR
+#endif
+
+/**
+ * Enable VPX VP8 codec.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_HAS_VPX_CODEC_VP8
+#   define PJMEDIA_HAS_VPX_CODEC_VP8            1
+#endif
+
+/**
+ * Enable VPX VP9 codec.
+ *
+ * Default: 0 (disabled)
+ */
+#ifndef PJMEDIA_HAS_VPX_CODEC_VP9
+#   define PJMEDIA_HAS_VPX_CODEC_VP9            0
+#endif
+
+/**
+ * Enable Android MediaCodec AMRNB codec.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_HAS_AND_MEDIA_AMRNB
+#   define PJMEDIA_HAS_AND_MEDIA_AMRNB          1
+#endif
+
+/**
+ * Enable Android MediaCodec AMRWB codec.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_HAS_AND_MEDIA_AMRWB
+#   define PJMEDIA_HAS_AND_MEDIA_AMRWB          1
+#endif
+
+/**
+ * Enable Android MediaCodec AVC/H264 codec.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_HAS_AND_MEDIA_H264
+#   define PJMEDIA_HAS_AND_MEDIA_H264           1
+#endif
+
+/**
+ * Enable Android MediaCodec VP8 codec.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_HAS_AND_MEDIA_VP8
+#   define PJMEDIA_HAS_AND_MEDIA_VP8            1
+#endif
+
+/**
+ * Enable Android MediaCodec VP9 codec.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_HAS_AND_MEDIA_VP9
+#   define PJMEDIA_HAS_AND_MEDIA_VP9            1
+#endif
+
+/**
+ * Prioritize to use software video encoder on Android MediaCodec.
+ * Set to 0 to prioritize Hardware encoder.
+ * Note: based on test, software encoder configuration provided the most stable
+ * configuration.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_AND_MEDIA_PRIO_SW_VID_ENC
+#    define PJMEDIA_AND_MEDIA_PRIO_SW_VID_ENC   1
+#endif
+
+/**
+ * Prioritize to use software video encoder on Android MediaCodec.
+ * Set to 0 to prioritize Hardware encoder.
+ * Note: based on test, software decoder configuration provided the most stable
+ * configuration.
+ *
+ * Default: 1
+ */
+#ifndef PJMEDIA_AND_MEDIA_PRIO_SW_VID_DEC
+#    define PJMEDIA_AND_MEDIA_PRIO_SW_VID_DEC   1
+#endif
+
+
+/**
+ * Maximum interval between keyframes for Apple VideoToolbox codecs,
+ * in second.
+ *
+ * Default: 5 (seconds)
+ */
+#ifndef PJMEDIA_CODEC_VID_TOOLBOX_MAX_KEYFRAME_INTERVAL
+#   define PJMEDIA_CODEC_VID_TOOLBOX_MAX_KEYFRAME_INTERVAL      5
+#endif
+
 /**
  * @}
  */
 
 
 
-#endif	/* __PJMEDIA_CODEC_CONFIG_H__ */
+#endif  /* __PJMEDIA_CODEC_CONFIG_H__ */
