@@ -24,6 +24,7 @@
  * @brief PJMEDIA main header file.
  */
 #include <pjmedia/alaw_ulaw.h>
+#include <pjmedia/avi.h>
 #include <pjmedia/avi_stream.h>
 #include <pjmedia/bidirectional.h>
 #include <pjmedia/circbuf.h>
@@ -67,6 +68,7 @@
 #include <pjmedia/transport_loop.h>
 #include <pjmedia/transport_srtp.h>
 #include <pjmedia/transport_udp.h>
+#include <pjmedia/txt_stream.h>
 #include <pjmedia/vid_codec.h>
 #include <pjmedia/vid_conf.h>
 #include <pjmedia/vid_port.h>
