@@ -218,6 +218,22 @@ typedef struct pj_turn_sock_tls_cfg
      */
     pj_ssl_cert_buffer privkey_buf;
 
+    /**
+     * Lookup certificate from OS certificate store with specified criteria.
+     *
+     * Currently only used by TLS backend Windows Schannel, please check
+     * pj_ssl_cert_load_from_store() for more info.
+     */
+    pj_ssl_cert_lookup_criteria cert_lookup;
+
+    /**
+     * TLS credentials using backend specific objects.
+     *
+     * Currently only used by TLS backend OpenSSL, please check
+     * pj_ssl_cert_load_direct() for more info.
+     */
+    pj_ssl_cert_direct cert_direct;
+
     /**
      * Password to open private key.
      */
@@ -355,8 +371,8 @@ typedef struct pj_turn_sock_cfg
     unsigned so_sndbuf_size;
 
     /**
-     * This specifies TLS settings for TLS transport. It is only be used
-     * when this TLS is used to connect to the TURN server.
+     * This specifies TLS settings for TLS transport. It's only applicable when
+     * TLS is used to connect to the TURN server.
      */
     pj_turn_sock_tls_cfg tls_cfg;
 
