@@ -298,6 +298,30 @@ struct AccountSipConfig : public PersistentObject
      */
     pjsua_ipv6_use      ipv6Use;
 
+    /**
+     * Use a shared authorization session within this account.
+     * This will use the accounts credentials on outgoing requests,
+     * so that less 401/407 Responses will be returned.
+     *
+     * Needs PJSIP_AUTH_AUTO_SEND_NEXT and PJSIP_AUTH_HEADER_CACHING
+     * enabled to work properly, and also will grow usage of the used pool for
+     * the cached headers.
+     *
+     * Default is disabled/false.
+     */
+    bool        useSharedAuth;
+
+    /**
+     * Configure automatic SIP-MESSAGE processing behavior for this account.
+     * When set to true, incoming SIP MESSAGE requests will be responded to
+     * immediately with a 200 OK response (legacy behavior). When set to false,
+     * incoming SIP MESSAGE requests will create UAS transactions allowing for
+     * deferred responses, cf. Account::sendResponse() and DeferredResponse.
+     *
+     * Default: true (automatic response for backward compatibility)
+     */
+    bool        autoRespondSipMessage;
+
 public:
     /**
      * Read this object from a container node.
@@ -344,6 +368,14 @@ struct AccountCallConfig : public PersistentObject
      */
     pjsua_sip_timer_use timerUse;
 
+    /**
+     * Specify the usage of SIPREC INVITE request. See the
+     * pjsua_sip_siprec_use for possible values.
+     * 
+     * Default: PJSUA_SIP_SIPREC_INACTIVE
+     */
+    pjsua_sip_siprec_use siprecUse;
+
     /**
      * Specify minimum Session Timer expiration period, in seconds.
      * Must not be lower than 90. Default is 90.
@@ -363,6 +395,7 @@ public:
     AccountCallConfig() : holdType(PJSUA_CALL_HOLD_TYPE_DEFAULT),
                           prackUse(PJSUA_100REL_NOT_USED),
                           timerUse(PJSUA_SIP_TIMER_OPTIONAL),
+                          siprecUse(PJSUA_SIP_SIPREC_INACTIVE),
                           timerMinSESec(90),
                           timerSessExpiresSec(PJSIP_SESS_TIMER_DEF_SE)
     {}
@@ -551,6 +584,14 @@ struct AccountNatConfig : public PersistentObject
      */
     int                 iceMaxHostCands;
 
+    /**
+     * Optional configuration to manually specify host candidates.
+     * Each candidate will use the same port as the automatic/base host
+     * candidate. The number of entries in this array must be equal or less
+     * than \a iceMaxHostCands.
+     */
+    SocketAddressVector iceManualHost;
+
     /**
      * Specify whether to use aggressive nomination.
      *
@@ -581,6 +622,15 @@ struct AccountNatConfig : public PersistentObject
      */
     int                 iceWaitNominationTimeoutMsec;
 
+    /**
+     * Specify whether to check the source address of the incoming messages.
+     * The source address will be compared to the remote candidate which has
+     * a completed connectivity check or received a connectivity check.
+     *
+     * Default value is PJ_ICE_SESS_CHECK_SRC_ADDR.
+     */
+    unsigned            iceCheckSrcAddr;
+
     /**
      * Disable RTCP component.
      *
@@ -769,6 +819,7 @@ public:
       iceAggressiveNomination(true),
       iceNominatedCheckDelayMsec(PJ_ICE_NOMINATED_CHECK_DELAY),
       iceWaitNominationTimeoutMsec(ICE_CONTROLLED_AGENT_WAIT_NOMINATION_TIMEOUT),
+      iceCheckSrcAddr(PJ_ICE_SESS_CHECK_SRC_ADDR),
       iceNoRtcp(false),
       iceAlwaysUpdate(true),
       turnEnabled(false),
@@ -828,8 +879,6 @@ public:
     SendRequestParam();
 };
 
-
-
 /**
  * SRTP crypto.
  */
@@ -1023,7 +1072,7 @@ public:
 };
 
 /**
- * Account media config (applicable for both audio and video). This will be
+ * Account media config (applicable for audio, video, and text). This will be
  * specified in AccountConfig.
  */
 struct AccountMediaConfig : public PersistentObject
@@ -1281,6 +1330,59 @@ public:
     virtual void writeObject(ContainerNode &node) const PJSUA2_THROW(Error);
 };
 
+/**
+ * Account text config. This will be specified in AccountConfig.
+ */
+struct AccountTextConfig : public PersistentObject
+{
+    /**
+     * Specifies text stream redundancy level, as specified in RFC 4103
+     * and 2198. When redundancy is enabled, each packet transmission
+     * will contain the current text data as well as a number of the
+     * previously transmitted text data to provide levels of redundancy.
+     * This mechanism offers protection against loss of data at the cost
+     * of additional bandwidth required.
+     *
+     * Value is integer indicating redundancy levels, i.e. the number
+     * of previous text data to be included with the current packet.
+     * (0 means disabled/no redundancy).
+     * A value of 1 provides an adequate protection against an average
+     * packet loss of up to 50%, while 2 can potentially protect
+     * against 66.7%.
+     * The maximum value is determined by PJMEDIA_TXT_STREAM_MAX_RED_LEVELS.
+     *
+     * Note that the redundancy level actually used is subject to remote
+     * capability and we will opt to use the lower redundancy value based
+     * on the result of SDP negotiation.
+     *
+     * Default: PJSUA_TXT_DEFAULT_REDUNDANCY_LEVEL (2), as per the
+     * recommendation of RFC 4103.
+     */
+    int              redundancyLevel;
+
+public:
+    /**
+     * Default constructor
+     */
+    AccountTextConfig()
+    : redundancyLevel(PJSUA_TXT_DEFAULT_REDUNDANCY_LEVEL)
+    {}
+
+    /**
+     * Read this object from a container node.
+     *
+     * @param node              Container to read values from.
+     */
+    virtual void readObject(const ContainerNode &node) PJSUA2_THROW(Error);
+
+    /**
+     * Write this object to a container node.
+     *
+     * @param node              Container to write values to.
+     */
+    virtual void writeObject(ContainerNode &node) const PJSUA2_THROW(Error);
+};
+
 /**
  * Account config specific to IP address change.
  */
@@ -1400,7 +1502,7 @@ struct AccountConfig : public PersistentObject
     AccountNatConfig    natConfig;
 
     /**
-     * Media settings (applicable for both audio and video).
+     * Media settings (applicable for audio, video, and text).
      */
     AccountMediaConfig  mediaConfig;
 
@@ -1409,6 +1511,11 @@ struct AccountConfig : public PersistentObject
      */
     AccountVideoConfig  videoConfig;
 
+    /**
+     * Text settings.
+     */
+    AccountTextConfig   textConfig;
+
     /**
      * IP Change settings.
      */
@@ -1841,6 +1948,86 @@ public:
     virtual ~FindBuddyMatch() {}
 };
 
+/**
+ * This class is used to store the received parsed SIP MESSAGE request and
+ * its transaction for use in sendResponse().
+ */
+class DeferredResponse
+{
+    friend class Account;
+public:
+    /**
+     * Constructor that clones the rx_data and keeps a non-owning pointer
+     * to the transaction.
+     * @param param        The instant message parameters from the callback.
+     */
+    explicit
+    DeferredResponse(const OnInstantMessageParam& param) PJSUA2_THROW(Error);
+
+    /**
+     * Constructs an empty object. Must be assigned to before use.
+     */
+    DeferredResponse();
+
+     /**
+      * Copies the deferred response object by cloning the rx_data.
+      */
+    DeferredResponse(const DeferredResponse& deferredResponse)
+        PJSUA2_THROW(Error);
+
+    /**
+     * Copies the deferred response object by cloning the rx_data.
+     */
+    DeferredResponse& operator=(const DeferredResponse& deferredResponse)
+        PJSUA2_THROW(Error);
+
+    /**
+     * Moves the deferred response object.
+     */
+    DeferredResponse(DeferredResponse&& deferredResponse) noexcept;
+
+    /**
+     * Moves the deferred response object.
+     */
+    DeferredResponse& operator=(DeferredResponse&& deferredResponse) noexcept;
+
+    /**
+     * Destructor that frees possible cloned rx_data.
+     */
+    virtual ~DeferredResponse();
+
+private:
+    /**
+     * Non-owning pointer to the registered transaction.
+     */
+    pjsip_transaction* transaction;
+
+    /**
+     * Owning pointer to the cloned rx_data.
+     */
+    pjsip_rx_data* data = nullptr;
+};
+
+/**
+ * Parameters for Account::sendResponse().
+ */
+struct SendResponseParam
+{
+    /**
+     * Deferred response object, which is valid until the response is sent.
+     */
+    DeferredResponse deferredResponse;
+
+    /**
+     * Status code of the response.
+     */
+    int         code;
+
+    /**
+     * Optional reason phrase of the response.
+     */
+    string      reason;
+};
 
 /**
  * Account.
@@ -1962,6 +2149,19 @@ public:
      */
     void sendRequest(const pj::SendRequestParam& prm) PJSUA2_THROW(Error);
 
+    /**
+     * Send response for incoming request that was deferred earlier
+     * using DeferredResponse object.
+     *
+     * @param prm                   The response's parameters.
+     * @param prm.deferredResponse  The deferred response object
+     *                              identifying the incoming request to
+     *                              respond to.
+     * @param prm.code              The status code of the response.
+     * @param prm.reason            Optional reason phrase of the response.
+     */
+    void sendResponse(const pj::SendResponseParam& prm) PJSUA2_THROW(Error);
+
     /**
      * Update registration or perform unregistration. Application normally
      * only needs to call this function if it wants to manually update the
