@@ -71,6 +71,12 @@ struct AuthCredInfo : public PersistentObject
      */
     string      data;
 
+
+    /**
+     * Digest algorithm type.
+     */
+    pjsip_auth_algorithm_type algoType;
+
     /*
      * Digest AKA credential information. Note that when AKA credential
      * is being used, the \a data field of this pjsip_cred_info is
@@ -126,6 +132,86 @@ public:
 
 //////////////////////////////////////////////////////////////////////////////
 
+/**
+ * Socket option type.
+ */
+struct SockOpt {
+    /**
+     * The level at which the option is defined.
+     */
+    int                 level;
+
+    /**
+     * Option name.
+     */
+    int                 optName;
+
+public:
+    /** Default constructor. */
+    SockOpt();
+
+    /** Construct a socket option with the specified parameters. */
+    SockOpt(int level, int optName, int optVal);
+
+    /**
+     * Set option value of type integer.
+     *
+     * @param opt_val           Option value.
+     */
+    void setOptValInt(int opt_val);
+
+private:
+    friend struct SockOptParams;
+
+    /** Pointer to the buffer in which the option is specified. */
+    void               *optVal;
+
+    /** Buffer size of the buffer pointed by optVal. */
+    int                 optLen;
+
+    /** Option value if the type is integer. */
+    int                 optValInt;
+};
+
+/** Array of socket options */
+typedef std::vector<SockOpt> SockOptVector;
+
+/**
+ * Socket option parameters, to be specified in TransportConfig.
+ */
+struct SockOptParams : public PersistentObject
+{
+    /**
+     * Array of socket options.
+     */
+    SockOptVector    sockOpts;
+
+public:
+    /** Default constructor initialises with default values */
+    SockOptParams();
+
+    /** Convert to pjsip */
+    pj_sockopt_params toPj() const;
+
+    /** Convert from pjsip */
+    void fromPj(const pj_sockopt_params &prm);
+
+    /**
+     * Read this object from a container node.
+     *
+     * @param node              Container to read values from.
+     */
+    virtual void readObject(const ContainerNode &node) PJSUA2_THROW(Error);
+
+    /**
+     * Write this object to a container node.
+     *
+     * @param node              Container to write values to.
+     */
+    virtual void writeObject(ContainerNode &node) const PJSUA2_THROW(Error);
+};
+
+
 /**
  * TLS transport settings, to be specified in TransportConfig.
  */
@@ -174,6 +260,51 @@ struct TlsConfig : public PersistentObject
      */
     string              privKeyBuf;
 
+    /**
+     * Lookup certificate from OS certificate store, this setting will
+     * specify the field type to lookup.
+     *
+     * Currently only used by Windows Schannel backend, see also
+     * \a pj_ssl_cert_load_from_store() for more info.
+     */
+    pj_ssl_cert_lookup_type certLookupType;
+
+    /**
+     * Lookup certificate from OS certificate store, this setting will
+     * specify the keyword to match on the field specified in
+     * \a certLookupType above.
+     *
+     * Currently only used by Windows Schannel backend, see also
+     * \a pj_ssl_cert_load_from_store() for more info.
+     */
+    string              certLookupKeyword;
+
+    /**
+     * When using backend specific credentials objects, specify the backend
+     * object types using this bit flag, see also \ref pj_ssl_cert_direct_type.
+     */
+    unsigned            credDirectType;
+
+    /**
+     * Optional backend specific private key. See also
+     * \a pj_ssl_cert_load_direct().
+     *
+     * Currently only the OpenSSL backend is supported. This setting
+     * corresponds to an OpenSSL EVP_PKEY instance. When this is set,
+     * specify PJ_SSL_CERT_DIRECT_OPENSSL_EVP_PKEY flag in \a certDirectType.
+     */
+    Token               privKeyDirect;
+
+    /**
+     * Optional backend specific certificate. See also
+     * \a pj_ssl_cert_load_direct().
+     *
+     * Currently only the OpenSSL backend is supported, this setting
+     * corresponds to an OpenSSL X509 instance. When this is set,
+     * specify PJ_SSL_CERT_DIRECT_OPENSSL_X509_CERT flag in \a certDirectType.
+     */
+    Token               certDirect;
+
     /**
      * TLS protocol method from #pjsip_ssl_method. In the future, this field
      * might be deprecated in favor of <b>proto</b> field. For now, this field 
@@ -281,6 +412,22 @@ struct TlsConfig : public PersistentObject
      */
     bool                qosIgnoreError;
 
+    /**
+     * Specify options to be set on the transport.
+     *
+     * By default, this is unset, which means that the underlying sockopt
+     * params as returned by #pj_ssl_sock_param_default() will be used.
+     */
+    SockOptParams       sockOptParams;
+
+    /**
+     * Specify if the transport should ignore any errors when setting the
+     * sockopt parameters.
+     *
+     * Default: true
+     */
+    bool                sockOptIgnoreError;
+
     /**
      * Specify if renegotiation is enabled for TLSv1.2 or earlier.
      *
@@ -376,9 +523,8 @@ struct TransportConfig : public PersistentObject
     string              boundAddress;
 
     /**
-     * This specifies TLS settings for TLS transport. It is only be used
-     * when this transport config is being used to create a SIP TLS
-     * transport.
+     * This specifies TLS settings for TLS transport. 
+     * It's only used when creating a SIP TLS transport.
      */
     TlsConfig           tlsConfig;
 
@@ -406,6 +552,16 @@ struct TransportConfig : public PersistentObject
      */
     pj_qos_params       qosParams;
 
+    /**
+     * Set the low level socket options to the transport.
+     *
+     * For TLS transport, this field will be ignored, the socket options
+     * can be set via tlsConfig.
+     *
+     * Default is no socket option set.
+     */
+    SockOptParams       sockOptParams;
+
 public:
     /** Default constructor initialises with default values */
     TransportConfig();
@@ -866,8 +1022,19 @@ struct SipTxOption
      */
     string                  localUri;
 
+    /**
+     * Optional contact URI to be used for this call. If empty (""), the
+     * contact will be generated automatically based on the account
+     * configuration. At the moment this field is only used when sending
+     * initial INVITE requests.
+     */
+    string                  contactUri;
+
     /**
      * Additional message headers to be included in the outgoing message.
+     *
+     * Application may override Max-Forwards header value (the default is
+     * #PJSIP_MAX_FORWARDS_VALUE) by adding a Max-Forwards header here.
      */
     SipHeaderVector         headers;
 
